@@ -29,7 +29,6 @@ static uint16_t get_pdu_session_reactivation_result(amf_ue_t *amf_ue);
 
 ogs_pkbuf_t *gmm_build_registration_accept(amf_ue_t *amf_ue)
 {
-    int i;
     int served_tai_index = 0;
     ogs_pkbuf_t *pkbuf = NULL;
 
@@ -42,6 +41,8 @@ ogs_pkbuf_t *gmm_build_registration_accept(amf_ue_t *amf_ue)
         &registration_accept->guti;
     ogs_nas_5gs_mobile_identity_guti_t mobile_identity_guti;
     ogs_nas_nssai_t *allowed_nssai = &registration_accept->allowed_nssai;
+    ogs_nas_rejected_nssai_t *rejected_nssai =
+        &registration_accept->rejected_nssai;
     ogs_nas_5gs_network_feature_support_t *network_feature_support =
         &registration_accept->network_feature_support;
     ogs_nas_pdu_session_status_t *pdu_session_status =
@@ -70,9 +71,9 @@ ogs_pkbuf_t *gmm_build_registration_accept(amf_ue_t *amf_ue)
 
     /* Set GUTI */
     ogs_debug("[%s]    %s 5G-S_GUTI[AMF_ID:0x%x,M_TMSI:0x%x]", amf_ue->supi,
-            amf_ue->guti_present ? "[V]" : "[N]",
+            amf_ue->guti_present == true ? "[V]" : "[N]",
             ogs_amf_id_hexdump(&amf_ue->guti.amf_id), amf_ue->guti.m_tmsi);
-    if (amf_ue->guti_present) {
+    if (amf_ue->guti_present == true) {
         registration_accept->presencemask |= OGS_NAS_5GS_REGISTRATION_ACCEPT_5G_GUTI_PRESENT;
 
         ogs_nas_5gs_nas_guti_to_mobilty_identity_guti(
@@ -81,18 +82,17 @@ ogs_pkbuf_t *gmm_build_registration_accept(amf_ue_t *amf_ue)
         mobile_identity->length = sizeof(mobile_identity_guti);
         mobile_identity->buffer = &mobile_identity_guti;
     }
-    amf_ue->guti_present = 0;
 
     /* Set TAI List */
     registration_accept->presencemask |= OGS_NAS_5GS_REGISTRATION_ACCEPT_TAI_LIST_PRESENT;
 
     ogs_debug("[%s]    TAI[PLMN_ID:%06x,TAC:%d]", amf_ue->supi,
-            ogs_plmn_id_hexdump(&amf_ue->tai.plmn_id), amf_ue->tai.tac.v);
+            ogs_plmn_id_hexdump(&amf_ue->nr_tai.plmn_id), amf_ue->nr_tai.tac.v);
     ogs_debug("[%s]    NR_CGI[PLMN_ID:%06x,CELL_ID:0x%llx]", amf_ue->supi,
             ogs_plmn_id_hexdump(&amf_ue->nr_cgi.plmn_id),
             (long long)amf_ue->nr_cgi.cell_id);
 
-    served_tai_index = amf_find_served_tai(&amf_ue->tai);
+    served_tai_index = amf_find_served_tai(&amf_ue->nr_tai);
     ogs_debug("[%s]    SERVED_TAI_INDEX[%d]", amf_ue->supi, served_tai_index);
     ogs_assert(served_tai_index >= 0 &&
             served_tai_index < OGS_MAX_NUM_OF_SERVED_TAI);
@@ -102,23 +102,25 @@ ogs_pkbuf_t *gmm_build_registration_accept(amf_ue_t *amf_ue)
             &amf_self()->served_tai[served_tai_index].list2);
 
     /* Set Allowed NSSAI */
-    allowed_nssai->length = 0;
-
-    for (i = 0; i < amf_self()->num_of_plmn_support; i++) {
-        if (memcmp(&amf_ue->tai.plmn_id,
-                &amf_self()->plmn_support[i].plmn_id, OGS_PLMN_ID_LEN) != 0)
-            continue;
-
-        ogs_debug("[%s]    NSSAI[PLMN_ID:%06x]", amf_ue->supi,
-                ogs_plmn_id_hexdump(&amf_self()->plmn_support[i].plmn_id));
+    if (amf_ue->allowed_nssai_present == true) {
+        ogs_assert(amf_ue->allowed_nssai.num_of_s_nssai);
 
         ogs_nas_build_nssai(allowed_nssai,
-            amf_self()->plmn_support[i].s_nssai,
-            amf_self()->plmn_support[i].num_of_s_nssai);
-    }
+                amf_ue->allowed_nssai.s_nssai,
+                amf_ue->allowed_nssai.num_of_s_nssai);
 
-    if (allowed_nssai->length) {
-        registration_accept->presencemask |= OGS_NAS_5GS_REGISTRATION_ACCEPT_ALLOWED_NSSAI_PRESENT;
+        registration_accept->presencemask |=
+            OGS_NAS_5GS_REGISTRATION_ACCEPT_ALLOWED_NSSAI_PRESENT;
+
+        if (amf_ue->rejected_nssai.num_of_s_nssai) {
+            ogs_nas_build_rejected_nssai(rejected_nssai,
+                    amf_ue->rejected_nssai.s_nssai,
+                    amf_ue->rejected_nssai.num_of_s_nssai);
+            registration_accept->presencemask |=
+                OGS_NAS_5GS_REGISTRATION_ACCEPT_REJECTED_NSSAI_PRESENT;
+        }
+
+        amf_ue->allowed_nssai_present = false;
     }
 
     /* 5GS network feature support */
@@ -388,29 +390,31 @@ ogs_pkbuf_t *gmm_build_security_mode_command(amf_ue_t *amf_ue)
     ngksi->tsc = amf_ue->nas.tsc;
     ngksi->value = amf_ue->nas.ksi;
 
-    replayed_ue_security_capabilities->nea = amf_ue->ue_security_capability.nea;
-    replayed_ue_security_capabilities->nia = amf_ue->ue_security_capability.nia;
-    replayed_ue_security_capabilities->eps_ea =
-        amf_ue->ue_security_capability.eps_ea;
-    replayed_ue_security_capabilities->eps_ia =
-        amf_ue->ue_security_capability.eps_ia;
+    replayed_ue_security_capabilities->nr_ea =
+        amf_ue->ue_security_capability.nr_ea;
+    replayed_ue_security_capabilities->nr_ia =
+        amf_ue->ue_security_capability.nr_ia;
+    replayed_ue_security_capabilities->eutra_ea =
+        amf_ue->ue_security_capability.eutra_ea;
+    replayed_ue_security_capabilities->eutra_ia =
+        amf_ue->ue_security_capability.eutra_ia;
 
     replayed_ue_security_capabilities->length =
-        sizeof(replayed_ue_security_capabilities->nea) +
-        sizeof(replayed_ue_security_capabilities->nia);
-    if (replayed_ue_security_capabilities->eps_ea ||
-        replayed_ue_security_capabilities->eps_ia)
+        sizeof(replayed_ue_security_capabilities->nr_ea) +
+        sizeof(replayed_ue_security_capabilities->nr_ia);
+    if (replayed_ue_security_capabilities->eutra_ea ||
+        replayed_ue_security_capabilities->eutra_ia)
         replayed_ue_security_capabilities->length =
-            sizeof(replayed_ue_security_capabilities->nea) +
-            sizeof(replayed_ue_security_capabilities->nia) +
-            sizeof(replayed_ue_security_capabilities->eps_ea) +
-            sizeof(replayed_ue_security_capabilities->eps_ia);
+            sizeof(replayed_ue_security_capabilities->nr_ea) +
+            sizeof(replayed_ue_security_capabilities->nr_ia) +
+            sizeof(replayed_ue_security_capabilities->eutra_ea) +
+            sizeof(replayed_ue_security_capabilities->eutra_ia);
     ogs_debug("    Replayed UE SEC[LEN:%d NEA:0x%x NIA:0x%x EEA:0x%x EIA:0x%x",
             replayed_ue_security_capabilities->length,
-            replayed_ue_security_capabilities->nea,
-            replayed_ue_security_capabilities->nia,
-            replayed_ue_security_capabilities->eps_ea,
-            replayed_ue_security_capabilities->eps_ia);
+            replayed_ue_security_capabilities->nr_ea,
+            replayed_ue_security_capabilities->nr_ia,
+            replayed_ue_security_capabilities->eutra_ea,
+            replayed_ue_security_capabilities->eutra_ia);
     ogs_debug("    Selected[Integrity:0x%x Encrypt:0x%x]",
             amf_ue->selected_int_algorithm, amf_ue->selected_enc_algorithm);
 
@@ -419,8 +423,7 @@ ogs_pkbuf_t *gmm_build_security_mode_command(amf_ue_t *amf_ue)
     imeisv_request->type = OGS_NAS_IMEISV_TYPE;
     imeisv_request->value = OGS_NAS_IMEISV_REQUESTED;
 
-    security_mode_command->presencemask |=
-        OGS_NAS_5GS_SECURITY_MODE_COMMAND_ADDITIONAL_5G_SECURITY_INFORMATION_PRESENT;
+    security_mode_command->presencemask |= OGS_NAS_5GS_SECURITY_MODE_COMMAND_ADDITIONAL_5G_SECURITY_INFORMATION_PRESENT;
     additional_security_information->length = 1;
     additional_security_information->
         retransmission_of_initial_nas_message_request = 1;
@@ -447,6 +450,8 @@ ogs_pkbuf_t *gmm_build_configuration_update_command(
     ogs_nas_5gs_configuration_update_command_t *configuration_update_command =
         &message.gmm.configuration_update_command;
 
+    ogs_nas_time_zone_t *local_time_zone =
+        &configuration_update_command->local_time_zone;
     ogs_nas_time_zone_and_time_t *universal_time_and_local_time_zone =
         &configuration_update_command->universal_time_and_local_time_zone;
     ogs_nas_daylight_saving_time_t *network_daylight_saving_time =
@@ -482,21 +487,45 @@ ogs_pkbuf_t *gmm_build_configuration_update_command(
     }
 
     if (param->nitz) {
+        if (amf_self()->full_name.length) {
+            configuration_update_command->presencemask |=
+                OGS_NAS_5GS_CONFIGURATION_UPDATE_COMMAND_FULL_NAME_FOR_NETWORK_PRESENT;
+            memcpy(&configuration_update_command->full_name_for_network,
+                &amf_self()->full_name, sizeof(ogs_nas_network_name_t));
+        }
+
+        if (amf_self()->short_name.length) {
+            configuration_update_command->presencemask |=
+                OGS_NAS_5GS_CONFIGURATION_UPDATE_COMMAND_SHORT_NAME_FOR_NETWORK_PRESENT;
+            memcpy(&configuration_update_command->short_name_for_network,
+                &amf_self()->short_name, sizeof(ogs_nas_network_name_t));
+        }
+
         ogs_gettimeofday(&tv);
         ogs_gmtime(tv.tv_sec, &gmt);
         ogs_localtime(tv.tv_sec, &local);
 
-        ogs_debug("    GMT Time[Y:M:D H:M:S GMT:DST] - "
-                    "%d:%d:%d, %d:%d:%d, %d:%d",
-            gmt.tm_year, gmt.tm_mon, gmt.tm_mday,
+        ogs_info("    UTC [%04d-%02d-%02dT%02d:%02d:%02d] "
+                "Timezone[%d]/DST[%d]",
+            gmt.tm_year+1900, gmt.tm_mon+1, gmt.tm_mday,
             gmt.tm_hour, gmt.tm_min, gmt.tm_sec,
             (int)gmt.tm_gmtoff, gmt.tm_isdst);
-        ogs_debug("    LOCAL Time[Y:M:D H:M:S GMT:DST] - "
-                    "%d:%d:%d, %d:%d:%d, %d:%d",
-            local.tm_year, local.tm_mon, local.tm_mday,
+        ogs_info("    LOCAL [%04d-%02d-%02dT%02d:%02d:%02d] "
+                "Timezone[%d]/DST[%d]",
+            local.tm_year+1900, local.tm_mon+1, local.tm_mday,
             local.tm_hour, local.tm_min, local.tm_sec,
             (int)local.tm_gmtoff, local.tm_isdst);
 
+        configuration_update_command->presencemask |=
+            OGS_NAS_5GS_CONFIGURATION_UPDATE_COMMAND_LOCAL_TIME_ZONE_PRESENT;
+        if (local.tm_gmtoff >= 0) {
+            *local_time_zone = OGS_NAS_TIME_TO_BCD(local.tm_gmtoff / 900);
+        } else {
+            *local_time_zone = OGS_NAS_TIME_TO_BCD((-local.tm_gmtoff) / 900);
+            *local_time_zone |= 0x08;
+        }
+        ogs_debug("    Timezone:0x%x", *local_time_zone);
+
         configuration_update_command->presencemask |=
             OGS_NAS_5GS_CONFIGURATION_UPDATE_COMMAND_UNIVERSAL_TIME_AND_LOCAL_TIME_ZONE_PRESENT;
         universal_time_and_local_time_zone->year =
@@ -511,34 +540,11 @@ ogs_pkbuf_t *gmm_build_configuration_update_command(
                     OGS_NAS_TIME_TO_BCD(gmt.tm_min);
         universal_time_and_local_time_zone->sec =
                     OGS_NAS_TIME_TO_BCD(gmt.tm_sec);
-        if (local.tm_gmtoff >= 0) {
-            universal_time_and_local_time_zone->timezone =
-                        OGS_NAS_TIME_TO_BCD(local.tm_gmtoff / 900);
-        } else {
-            universal_time_and_local_time_zone->timezone =
-                        OGS_NAS_TIME_TO_BCD((-local.tm_gmtoff) / 900);
-            universal_time_and_local_time_zone->timezone |= 0x08;
-        }
-        ogs_debug("    Timezone:0x%x",
-            universal_time_and_local_time_zone->timezone);
+        universal_time_and_local_time_zone->timezone = *local_time_zone;
 
         configuration_update_command->presencemask |=
             OGS_NAS_5GS_CONFIGURATION_UPDATE_COMMAND_NETWORK_DAYLIGHT_SAVING_TIME_PRESENT;
         network_daylight_saving_time->length = 1;
-
-        if (amf_self()->full_name.length) {
-            configuration_update_command->presencemask |=
-                OGS_NAS_5GS_CONFIGURATION_UPDATE_COMMAND_FULL_NAME_FOR_NETWORK_PRESENT;
-            memcpy(&configuration_update_command->full_name_for_network,
-                &amf_self()->full_name, sizeof(ogs_nas_network_name_t));
-        }
-
-        if (amf_self()->short_name.length) {
-            configuration_update_command->presencemask |=
-                OGS_NAS_5GS_CONFIGURATION_UPDATE_COMMAND_SHORT_NAME_FOR_NETWORK_PRESENT;
-            memcpy(&configuration_update_command->short_name_for_network,
-                &amf_self()->short_name, sizeof(ogs_nas_network_name_t));
-        }
     }
 
     return nas_5gs_security_encode(amf_ue, &message);
@@ -645,12 +651,12 @@ ogs_pkbuf_t *gmm_build_tau_accept(amf_ue_t *amf_ue)
         OGS_NAS_5GS_TRACKING_AREA_UPDATE_ACCEPT_TAI_LIST_PRESENT;
 
     ogs_debug("    TAI[PLMN_ID:%06x,TAC:%d]",
-            ogs_plmn_id_hexdump(&amf_ue->tai.plmn_id),
-            amf_ue->tai.tac);
+            ogs_plmn_id_hexdump(&amf_ue->nr_tai.plmn_id),
+            amf_ue->nr_tai.tac);
     ogs_debug("    E_CGI[PLMN_ID:%06x,CELL_ID:%d]",
             ogs_plmn_id_hexdump(&amf_ue->e_cgi.plmn_id),
             amf_ue->e_cgi.cell_id);
-    served_tai_index = amf_find_served_tai(&amf_ue->tai);
+    served_tai_index = amf_find_served_tai(&amf_ue->nr_tai);
     ogs_debug("    SERVED_TAI_INDEX[%d]", served_tai_index);
     ogs_assert(served_tai_index >= 0 &&
             served_tai_index < OGS_MAX_NUM_OF_SERVED_TAI);
