@@ -56,8 +56,6 @@ int gmm_handle_registration_request(amf_ue_t *amf_ue,
     ue_security_capability = &registration_request->ue_security_capability;
     ogs_assert(ue_security_capability);
 
-    ogs_debug("Registration request");
-
     if (!mobile_identity->length || !mobile_identity->buffer) {
         ogs_error("No Mobile Identity");
         nas_5gs_send_registration_reject(amf_ue,
@@ -71,7 +69,7 @@ int gmm_handle_registration_request(amf_ue_t *amf_ue,
     switch (mobile_identity_header->type) {
     case OGS_NAS_5GS_MOBILE_IDENTITY_SUCI:
         amf_ue_set_suci(amf_ue, mobile_identity);
-        ogs_debug("[%s]    SUCI", amf_ue->suci);
+        ogs_info("[%s]    SUCI", amf_ue->suci);
         break;
     case OGS_NAS_5GS_MOBILE_IDENTITY_GUTI:
         mobile_identity_guti =
@@ -84,7 +82,7 @@ int gmm_handle_registration_request(amf_ue_t *amf_ue,
         ogs_nas_5gs_mobile_identity_guti_to_nas_guti(
             mobile_identity_guti, &nas_guti);
 
-        ogs_debug("[%s]    5G-S_GUTI[AMF_ID:0x%x,M_TMSI:0x%x]",
+        ogs_info("[%s]    5G-S_GUTI[AMF_ID:0x%x,M_TMSI:0x%x]",
             AMF_UE_HAVE_SUCI(amf_ue) ? amf_ue->suci : "Unknown ID",
             ogs_amf_id_hexdump(&nas_guti.amf_id), nas_guti.m_tmsi);
         break;
@@ -104,8 +102,11 @@ int gmm_handle_registration_request(amf_ue_t *amf_ue,
     /*
      * REGISTRATION_REQUEST
      * SERVICE_REQUEST
+     *   Clear N2 Transfer
      *   Clear Timer and Message
      */
+    AMF_UE_CLEAR_PAGING_INFO(amf_ue);
+    AMF_UE_CLEAR_N2_TRANSFER(amf_ue, pdu_session_resource_setup_request);
     CLEAR_AMF_UE_ALL_TIMERS(amf_ue);
 
     if (SECURITY_CONTEXT_IS_VALID(amf_ue)) {
@@ -117,28 +118,29 @@ int gmm_handle_registration_request(amf_ue_t *amf_ue,
     }
 
     ogs_debug("    OLD TAI[PLMN_ID:%06x,TAC:%d]",
-            ogs_plmn_id_hexdump(&amf_ue->tai.plmn_id), amf_ue->tai.tac.v);
+            ogs_plmn_id_hexdump(&amf_ue->nr_tai.plmn_id), amf_ue->nr_tai.tac.v);
     ogs_debug("    OLD NR_CGI[PLMN_ID:%06x,CELL_ID:0x%llx]",
             ogs_plmn_id_hexdump(&amf_ue->nr_cgi.plmn_id),
             (long long)amf_ue->nr_cgi.cell_id);
     ogs_debug("    TAI[PLMN_ID:%06x,TAC:%d]",
-            ogs_plmn_id_hexdump(&ran_ue->saved.tai.plmn_id),
-            ran_ue->saved.tai.tac.v);
+            ogs_plmn_id_hexdump(&ran_ue->saved.nr_tai.plmn_id),
+            ran_ue->saved.nr_tai.tac.v);
     ogs_debug("    NR_CGI[PLMN_ID:%06x,CELL_ID:0x%llx]",
             ogs_plmn_id_hexdump(&ran_ue->saved.nr_cgi.plmn_id),
             (long long)ran_ue->saved.nr_cgi.cell_id);
 
-    /* Copy TAI and ECGI from ran_ue */
-    memcpy(&amf_ue->tai, &ran_ue->saved.tai, sizeof(ogs_5gs_tai_t));
+    /* Copy Stream-No/NR-TAI/NR-CGI from ran_ue */
+    amf_ue->gnb_ostream_id = ran_ue->gnb_ostream_id;
+    memcpy(&amf_ue->nr_tai, &ran_ue->saved.nr_tai, sizeof(ogs_5gs_tai_t));
     memcpy(&amf_ue->nr_cgi, &ran_ue->saved.nr_cgi, sizeof(ogs_nr_cgi_t));
     amf_ue->ue_location_timestamp = ogs_time_now();
 
     /* Check TAI */
-    served_tai_index = amf_find_served_tai(&amf_ue->tai);
+    served_tai_index = amf_find_served_tai(&amf_ue->nr_tai);
     if (served_tai_index < 0) {
         /* Send Registration Reject */
         ogs_warn("Cannot find Served TAI[PLMN_ID:%06x,TAC:%d]",
-            ogs_plmn_id_hexdump(&amf_ue->tai.plmn_id), amf_ue->tai.tac.v);
+            ogs_plmn_id_hexdump(&amf_ue->nr_tai.plmn_id), amf_ue->nr_tai.tac.v);
         nas_5gs_send_registration_reject(amf_ue,
             OGS_5GMM_CAUSE_TRACKING_AREA_NOT_ALLOWED);
         return OGS_ERROR;
@@ -181,7 +183,7 @@ int gmm_handle_registration_request(amf_ue_t *amf_ue,
             OGS_NAS_SECURITY_ALGORITHMS_NIA0) {
         ogs_error("[UE:0x%x:0x%x], NEA0 can be used in Encrypt[0x%x], "
             "but Integrity[0x%x] cannot be bypassed with NIA0",
-            ue_security_capability->nea, ue_security_capability->nia,
+            ue_security_capability->nr_ea, ue_security_capability->nr_ia,
             amf_selected_enc_algorithm(amf_ue), 
             amf_selected_int_algorithm(amf_ue));
         nas_5gs_send_registration_reject(amf_ue,
@@ -224,24 +226,27 @@ int gmm_handle_registration_update(amf_ue_t *amf_ue,
     if (registration_request->presencemask &
         OGS_NAS_5GS_REGISTRATION_REQUEST_REQUESTED_NSSAI_PRESENT) {
 
-        amf_ue->num_of_requested_nssai = ogs_nas_parse_nssai(
-            amf_ue->requested_nssai, &registration_request->requested_nssai);
+        amf_ue->requested_nssai.num_of_s_nssai =
+            ogs_nas_parse_nssai(
+                    amf_ue->requested_nssai.s_nssai,
+                    &registration_request->requested_nssai);
 
-        for (i = 0; i < amf_ue->num_of_requested_nssai; i++) {
+        for (i = 0; i < amf_ue->requested_nssai.num_of_s_nssai; i++) {
             if (amf_find_s_nssai(
-                    &amf_ue->tai.plmn_id, &amf_ue->requested_nssai[i]))
+                    &amf_ue->nr_tai.plmn_id,
+                    (ogs_s_nssai_t *)&amf_ue->requested_nssai.s_nssai[i]))
                 break;
         }
 
-        if (i == amf_ue->num_of_requested_nssai) {
+        if (i == amf_ue->requested_nssai.num_of_s_nssai) {
             ogs_error("CHECK CONFIGURATION: Cannot find Requested NSSAI");
-            for (i = 0; i < amf_ue->num_of_requested_nssai; i++) {
+            for (i = 0; i < amf_ue->requested_nssai.num_of_s_nssai; i++) {
                 ogs_error("    PLMN_ID[MCC:%d MNC:%d]",
-                        ogs_plmn_id_mcc(&amf_ue->tai.plmn_id),
-                        ogs_plmn_id_mnc(&amf_ue->tai.plmn_id));
+                        ogs_plmn_id_mcc(&amf_ue->nr_tai.plmn_id),
+                        ogs_plmn_id_mnc(&amf_ue->nr_tai.plmn_id));
                 ogs_error("    S_NSSAI[SST:%d SD:0x%x]",
-                        amf_ue->requested_nssai[i].sst,
-                        amf_ue->requested_nssai[i].sd.v);
+                        amf_ue->requested_nssai.s_nssai[i].sst,
+                        amf_ue->requested_nssai.s_nssai[i].sd.v);
             }
             return OGS_ERROR;
         }
@@ -271,36 +276,6 @@ int gmm_handle_registration_update(amf_ue_t *amf_ue,
         ogs_error("Not implemented for Allowed PDU Session Status IE");
     }
 
-    if ((registration_request->presencemask &
-        OGS_NAS_5GS_REGISTRATION_REQUEST_UPLINK_DATA_STATUS_PRESENT) == 0) {
-        amf_ue->nas.present.uplink_data_status = 0;
-    } else {
-        amf_ue->nas.present.uplink_data_status = 1;
-
-        psimask = 0;
-        psimask |= uplink_data_status->psi << 8;
-        psimask |= uplink_data_status->psi >> 8;
-
-        ogs_list_for_each(&amf_ue->sess_list, sess) {
-            if (psimask & (1 << sess->psi)) {
-#if REMOVED
-    /*
-     * TS23.502
-     * 4.2.3.2 UE Triggered Service Request
-     *
-     * Step 4. The Nsmf_PDUSession_UpdateSMContext Request is invoked:
-     *
-     *  - If the UE identifies List Of PDU Sessions To Be Activated
-     *    in the Service Request message;
-     *          if (sess->smfUpCnxState == OpenAPI_up_cnx_state_DEACTIVATED)
-     */
-#endif
-                if (SESSION_CONTEXT_IN_SMF(sess))
-                    amf_sbi_send_activating_session(sess);
-            }
-        }
-    }
-
     if ((registration_request->presencemask &
             OGS_NAS_5GS_REGISTRATION_REQUEST_PDU_SESSION_STATUS_PRESENT) == 0) {
         amf_ue->nas.present.pdu_session_status = 0;
@@ -320,6 +295,25 @@ int gmm_handle_registration_update(amf_ue_t *amf_ue,
         }
     }
 
+    if ((registration_request->presencemask &
+        OGS_NAS_5GS_REGISTRATION_REQUEST_UPLINK_DATA_STATUS_PRESENT) == 0) {
+        amf_ue->nas.present.uplink_data_status = 0;
+    } else {
+        amf_ue->nas.present.uplink_data_status = 1;
+
+        psimask = 0;
+        psimask |= uplink_data_status->psi << 8;
+        psimask |= uplink_data_status->psi >> 8;
+
+        ogs_list_for_each(&amf_ue->sess_list, sess) {
+            if (psimask & (1 << sess->psi)) {
+                if (SESSION_CONTEXT_IN_SMF(sess))
+                    amf_sbi_send_activating_session(
+                            sess, AMF_UPDATE_SM_CONTEXT_REGISTRATION_REQUEST);
+            }
+        }
+    }
+
     return OGS_OK;
 }
 
@@ -338,8 +332,6 @@ int gmm_handle_service_request(amf_ue_t *amf_ue,
     ngksi = &service_request->ngksi;
     ogs_assert(ngksi);
 
-    ogs_debug("Service request");
-
     amf_ue->nas.message_type = OGS_NAS_5GS_SERVICE_REQUEST;
     amf_ue->nas.tsc = ngksi->tsc;
     amf_ue->nas.ksi = ngksi->value;
@@ -350,8 +342,10 @@ int gmm_handle_service_request(amf_ue_t *amf_ue,
     /*
      * REGISTRATION_REQUEST
      * SERVICE_REQUEST
+     *   Clear Paging Info
      *   Clear Timer and Message
      */
+    AMF_UE_CLEAR_PAGING_INFO(amf_ue);
     CLEAR_AMF_UE_ALL_TIMERS(amf_ue);
 
     if (SECURITY_CONTEXT_IS_VALID(amf_ue)) {
@@ -363,35 +357,36 @@ int gmm_handle_service_request(amf_ue_t *amf_ue,
     }
 
     ogs_debug("    OLD TAI[PLMN_ID:%06x,TAC:%d]",
-            ogs_plmn_id_hexdump(&amf_ue->tai.plmn_id), amf_ue->tai.tac.v);
+            ogs_plmn_id_hexdump(&amf_ue->nr_tai.plmn_id), amf_ue->nr_tai.tac.v);
     ogs_debug("    OLD NR_CGI[PLMN_ID:%06x,CELL_ID:0x%llx]",
             ogs_plmn_id_hexdump(&amf_ue->nr_cgi.plmn_id),
             (long long)amf_ue->nr_cgi.cell_id);
     ogs_debug("    TAI[PLMN_ID:%06x,TAC:%d]",
-            ogs_plmn_id_hexdump(&ran_ue->saved.tai.plmn_id),
-            ran_ue->saved.tai.tac.v);
+            ogs_plmn_id_hexdump(&ran_ue->saved.nr_tai.plmn_id),
+            ran_ue->saved.nr_tai.tac.v);
     ogs_debug("    NR_CGI[PLMN_ID:%06x,CELL_ID:0x%llx]",
             ogs_plmn_id_hexdump(&ran_ue->saved.nr_cgi.plmn_id),
             (long long)ran_ue->saved.nr_cgi.cell_id);
 
-    /* Copy TAI and ECGI from ran_ue */
-    memcpy(&amf_ue->tai, &ran_ue->saved.tai, sizeof(ogs_5gs_tai_t));
+    /* Copy Stream-No/NR-TAI/NR-CGI from ran_ue */
+    amf_ue->gnb_ostream_id = ran_ue->gnb_ostream_id;
+    memcpy(&amf_ue->nr_tai, &ran_ue->saved.nr_tai, sizeof(ogs_5gs_tai_t));
     memcpy(&amf_ue->nr_cgi, &ran_ue->saved.nr_cgi, sizeof(ogs_nr_cgi_t));
     amf_ue->ue_location_timestamp = ogs_time_now();
 
     /* Check TAI */
-    served_tai_index = amf_find_served_tai(&amf_ue->tai);
+    served_tai_index = amf_find_served_tai(&amf_ue->nr_tai);
     if (served_tai_index < 0) {
         /* Send Registration Reject */
         ogs_warn("Cannot find Served TAI[PLMN_ID:%06x,TAC:%d]",
-            ogs_plmn_id_hexdump(&amf_ue->tai.plmn_id), amf_ue->tai.tac.v);
+            ogs_plmn_id_hexdump(&amf_ue->nr_tai.plmn_id), amf_ue->nr_tai.tac.v);
         nas_5gs_send_registration_reject(amf_ue,
             OGS_5GMM_CAUSE_TRACKING_AREA_NOT_ALLOWED);
         return OGS_ERROR;
     }
     ogs_debug("    SERVED_TAI_INDEX[%d]", served_tai_index);
 
-    ogs_debug("[%s]    5G-S_GUTI[AMF_ID:0x%x,M_TMSI:0x%x]",
+    ogs_info("[%s]    5G-S_GUTI[AMF_ID:0x%x,M_TMSI:0x%x]",
         AMF_UE_HAVE_SUCI(amf_ue) ? amf_ue->suci : "Unknown ID",
         ogs_amf_id_hexdump(&amf_ue->guti.amf_id), amf_ue->guti.m_tmsi);
 
@@ -427,44 +422,6 @@ int gmm_handle_service_update(amf_ue_t *amf_ue,
 
     xact_count = amf_sess_xact_count(amf_ue);
 
-    if ((service_request->presencemask &
-        OGS_NAS_5GS_SERVICE_REQUEST_ALLOWED_PDU_SESSION_STATUS_PRESENT) == 0) {
-        amf_ue->nas.present.allowed_pdu_session_status = 0;
-    } else {
-        amf_ue->nas.present.allowed_pdu_session_status = 1;
-        ogs_error("Not implemented for Allowed PDU Session Status IE");
-    }
-
-    if ((service_request->presencemask &
-            OGS_NAS_5GS_SERVICE_REQUEST_UPLINK_DATA_STATUS_PRESENT) == 0) {
-        amf_ue->nas.present.uplink_data_status = 0;
-    } else {
-        amf_ue->nas.present.uplink_data_status = 1;
-
-        psimask = 0;
-        psimask |= uplink_data_status->psi << 8;
-        psimask |= uplink_data_status->psi >> 8;
-
-        ogs_list_for_each(&amf_ue->sess_list, sess) {
-            if (psimask & (1 << sess->psi)) {
-#if REMOVED
-    /*
-     * TS23.502
-     * 4.2.3.2 UE Triggered Service Request
-     *
-     * Step 4. The Nsmf_PDUSession_UpdateSMContext Request is invoked:
-     *
-     *  - If the UE identifies List Of PDU Sessions To Be Activated
-     *    in the Service Request message;
-     *          if (sess->smfUpCnxState == OpenAPI_up_cnx_state_DEACTIVATED)
-     */
-#endif
-                if (SESSION_CONTEXT_IN_SMF(sess))
-                    amf_sbi_send_activating_session(sess);
-            }
-        }
-    }
-
     /*
      * TS24.501
      * 5.6.1.5 Service request procedure not accepted by the network
@@ -499,6 +456,33 @@ int gmm_handle_service_update(amf_ue_t *amf_ue,
         }
     }
 
+    if ((service_request->presencemask &
+        OGS_NAS_5GS_SERVICE_REQUEST_ALLOWED_PDU_SESSION_STATUS_PRESENT) == 0) {
+        amf_ue->nas.present.allowed_pdu_session_status = 0;
+    } else {
+        amf_ue->nas.present.allowed_pdu_session_status = 1;
+        ogs_error("Not implemented for Allowed PDU Session Status IE");
+    }
+
+    if ((service_request->presencemask &
+            OGS_NAS_5GS_SERVICE_REQUEST_UPLINK_DATA_STATUS_PRESENT) == 0) {
+        amf_ue->nas.present.uplink_data_status = 0;
+    } else {
+        amf_ue->nas.present.uplink_data_status = 1;
+
+        psimask = 0;
+        psimask |= uplink_data_status->psi << 8;
+        psimask |= uplink_data_status->psi >> 8;
+
+        ogs_list_for_each(&amf_ue->sess_list, sess) {
+            if (psimask & (1 << sess->psi)) {
+                if (SESSION_CONTEXT_IN_SMF(sess))
+                    amf_sbi_send_activating_session(
+                            sess, AMF_UPDATE_SM_CONTEXT_SERVICE_REQUEST);
+            }
+        }
+    }
+
     if (amf_sess_xact_count(amf_ue) == xact_count)
         nas_5gs_send_service_accept(amf_ue);
 
@@ -515,8 +499,6 @@ int gmm_handle_deregistration_request(amf_ue_t *amf_ue,
 
     de_registration_type = &deregistration_request->de_registration_type;
 
-    ogs_debug("[%s] Deregistration request", amf_ue->supi);
-
     /* Set 5GS Attach Type */
     memcpy(&amf_ue->nas.de_registration,
             de_registration_type, sizeof(ogs_nas_de_registration_type_t));
@@ -531,8 +513,9 @@ int gmm_handle_deregistration_request(amf_ue_t *amf_ue,
     if (deregistration_request->de_registration_type.switch_off)
         ogs_debug("    Switch-Off");
 
-    amf_sbi_send_release_all_sessions(
-            amf_ue, AMF_RELEASE_SM_CONTEXT_NO_STATE);
+    ogs_info("[%s]    SUCI", amf_ue->suci);
+
+    amf_sbi_send_release_all_sessions(amf_ue, AMF_RELEASE_SM_CONTEXT_NO_STATE);
 
     if (ogs_list_count(&amf_ue->sess_list) == 0)
         nas_5gs_send_de_registration_accept(amf_ue);
@@ -612,7 +595,7 @@ int gmm_handle_identity_response(amf_ue_t *amf_ue,
 
     if (mobile_identity_header->type == OGS_NAS_5GS_MOBILE_IDENTITY_SUCI) {
         amf_ue_set_suci(amf_ue, mobile_identity);
-        ogs_debug("[%s]    SUCI", amf_ue->suci);
+        ogs_info("[%s]    SUCI", amf_ue->suci);
     } else {
         ogs_error("Not supported Identity type[%d]",
                 mobile_identity_header->type);
@@ -641,17 +624,31 @@ int gmm_handle_security_mode_complete(amf_ue_t *amf_ue,
 
         switch (mobile_identity_imeisv->type) {
         case OGS_NAS_5GS_MOBILE_IDENTITY_IMEISV:
-            memcpy(&amf_ue->nas_mobile_identity_imeisv,
-                mobile_identity_imeisv, imeisv->length);
-            ogs_nas_imeisv_to_bcd(mobile_identity_imeisv, imeisv->length,
-                    amf_ue->imeisv_bcd);
-            ogs_nas_imeisv_bcd_to_buffer(amf_ue->imeisv_bcd,
-                    amf_ue->masked_imeisv, &amf_ue->masked_imeisv_len);
-            amf_ue->masked_imeisv[5] = 0xff;
-            amf_ue->masked_imeisv[6] = 0xff;
-            if (amf_ue->pei)
-                ogs_free(amf_ue->pei);
-            amf_ue->pei = ogs_msprintf("imeisv-%s", amf_ue->imeisv_bcd);
+            /* TS23.003 6.2.2 Composition of IMEISV
+             *
+             * The International Mobile station Equipment Identity and
+             * Software Version Number (IMEISV) is composed.
+             *
+             * TAC(8 digits) - SNR(6 digits) - SVN(2 digits)
+             * IMEISV(16 digits) ==> 8bytes
+             */
+            if (imeisv->length == sizeof(ogs_nas_mobile_identity_imeisv_t)) {
+                memcpy(&amf_ue->nas_mobile_identity_imeisv,
+                    mobile_identity_imeisv, imeisv->length);
+                ogs_nas_imeisv_to_bcd(mobile_identity_imeisv, imeisv->length,
+                        amf_ue->imeisv_bcd);
+                ogs_nas_imeisv_bcd_to_buffer(amf_ue->imeisv_bcd,
+                        amf_ue->masked_imeisv, &amf_ue->masked_imeisv_len);
+                amf_ue->masked_imeisv[5] = 0xff;
+                amf_ue->masked_imeisv[6] = 0xff;
+                if (amf_ue->pei)
+                    ogs_free(amf_ue->pei);
+                amf_ue->pei = ogs_msprintf("imeisv-%s", amf_ue->imeisv_bcd);
+            } else {
+                ogs_error("[%s] Unknown IMEISV Length [%d]",
+                        amf_ue->supi, imeisv->length);
+                ogs_log_hexdump(OGS_LOG_ERROR, imeisv->buffer, imeisv->length);
+            }
             break;
         default:
             ogs_warn("[%s] Invalid IMEISV Type [%d]",
@@ -674,14 +671,14 @@ int gmm_handle_security_mode_complete(amf_ue_t *amf_ue,
 int gmm_handle_ul_nas_transport(amf_ue_t *amf_ue,
         ogs_nas_5gs_ul_nas_transport_t *ul_nas_transport)
 {
+    ogs_slice_data_t *selected_slice = NULL;
     amf_sess_t *sess = NULL;
-    amf_nsmf_pdu_session_update_sm_context_param_t param;
+    amf_nsmf_pdusession_update_sm_context_param_t param;
 
     ogs_nas_payload_container_type_t *payload_container_type = NULL;
     ogs_nas_payload_container_t *payload_container = NULL;
     ogs_nas_pdu_session_identity_2_t *pdu_session_id = NULL;
     ogs_nas_s_nssai_t *nas_s_nssai = NULL;
-    ogs_s_nssai_t *selected_s_nssai = NULL;
     ogs_nas_dnn_t *dnn = NULL;
     ogs_nas_5gsm_header_t *gsm_header = NULL;
 
@@ -774,26 +771,33 @@ int gmm_handle_ul_nas_transport(amf_ue_t *amf_ue,
 
             if (ul_nas_transport->presencemask &
                     OGS_NAS_5GS_UL_NAS_TRANSPORT_S_NSSAI_PRESENT) {
-                ogs_s_nssai_t s_nssai;
-                if (ogs_nas_parse_s_nssai(&s_nssai, nas_s_nssai) != 0) {
-                    selected_s_nssai =
-                        amf_find_s_nssai(&amf_ue->tai.plmn_id, &s_nssai);
+                ogs_nas_s_nssai_ie_t ie;
+                if (ogs_nas_parse_s_nssai(&ie, nas_s_nssai) != 0) {
+                    selected_slice = ogs_slice_find_by_s_nssai(
+                                amf_ue->slice, amf_ue->num_of_slice,
+                                (ogs_s_nssai_t *)&ie);
                 }
             }
 
-            if (!selected_s_nssai) {
-                if (amf_ue->num_of_requested_nssai) {
-                    selected_s_nssai = &amf_ue->requested_nssai[0];
+            if (!selected_slice) {
+                int i;
+                for (i = 0; i < amf_ue->num_of_slice; i++) {
+                    if (amf_ue->slice[i].default_indicator == true) {
+                        selected_slice = &amf_ue->slice[i];
+                    }
                 }
             }
 
-            if (!selected_s_nssai) {
-                ogs_error("No S_NSSAI : Set default S_NSSAI using AMF config");
-                selected_s_nssai = &amf_self()->plmn_support[0].s_nssai[0];
-                ogs_assert(selected_s_nssai);
+            if (!selected_slice) {
+                ogs_error("[%s] No S-NSSAI", amf_ue->supi);
+                nas_5gs_send_gmm_status(amf_ue,
+                    OGS_5GMM_CAUSE_INSUFFICIENT_RESOURCES_FOR_SPECIFIC_SLICE);
+                return OGS_ERROR;
             }
 
-            memcpy(&sess->s_nssai, selected_s_nssai, sizeof(ogs_s_nssai_t));
+            /* Store S-NSSAI */
+            sess->s_nssai.sst = selected_slice->s_nssai.sst;
+            sess->s_nssai.sd.v = selected_slice->s_nssai.sd.v;
 
             if (ul_nas_transport->presencemask &
                     OGS_NAS_5GS_UL_NAS_TRANSPORT_DNN_PRESENT) {
@@ -802,9 +806,52 @@ int gmm_handle_ul_nas_transport(amf_ue_t *amf_ue,
                 sess->dnn = ogs_strdup(dnn->value);
             }
 
-            amf_sess_sbi_discover_and_send(OpenAPI_nf_type_SMF,
-                    sess, AMF_UPDATE_SM_CONTEXT_NO_STATE, NULL,
-                    amf_nsmf_pdu_session_build_create_sm_context);
+            if (!sess->dnn) {
+                if (selected_slice->num_of_session) {
+                    sess->dnn = ogs_strdup(selected_slice->session[0].name);
+                }
+            }
+
+            if (!sess->dnn) {
+                ogs_error("[%s] No DNN", amf_ue->supi);
+                nas_5gs_send_gmm_status(amf_ue, OGS_5GMM_CAUSE_DNN_NOT_SUPPORTED_OR_NOT_SUBSCRIBED_IN_THE_SLICE);
+                return OGS_ERROR;
+            }
+
+            ogs_info("UE SUPI[%s] DNN[%s] S_NSSAI[SST:%d SD:0x%x]",
+                amf_ue->supi, sess->dnn, sess->s_nssai.sst, sess->s_nssai.sd.v);
+
+
+            if (!SESSION_CONTEXT_IN_SMF(sess)) {
+                ogs_sbi_nf_instance_t *nf_instance = NULL;
+
+                nf_instance = OGS_SBI_NF_INSTANCE(
+                                &sess->sbi, OpenAPI_nf_type_SMF);
+                if (!nf_instance) {
+                    amf_sess_select_smf(sess);
+                    nf_instance = OGS_SBI_NF_INSTANCE(
+                                    &sess->sbi, OpenAPI_nf_type_SMF);
+                }
+
+                if (nf_instance) {
+                    amf_sess_sbi_discover_and_send(OpenAPI_nf_type_SMF,
+                            sess, AMF_CREATE_SM_CONTEXT_NO_STATE, NULL,
+                            amf_nsmf_pdusession_build_create_sm_context);
+                } else {
+                    amf_sess_sbi_discover_and_send(OpenAPI_nf_type_NSSF,
+                            sess, 0, NULL, amf_nnssf_nsselection_build_get);
+                }
+
+            } else {
+
+                memset(&param, 0, sizeof(param));
+                param.release = 1;
+                param.cause = OpenAPI_cause_REL_DUE_TO_DUPLICATE_SESSION_ID;
+
+                amf_sess_sbi_discover_and_send(OpenAPI_nf_type_SMF,
+                        sess, AMF_UPDATE_SM_CONTEXT_DUPLICATED_PDU_SESSION_ID,
+                        &param, amf_nsmf_pdusession_build_update_sm_context);
+            }
 
         } else {
 
@@ -827,7 +874,7 @@ int gmm_handle_ul_nas_transport(amf_ue_t *amf_ue,
 
             amf_sess_sbi_discover_and_send(OpenAPI_nf_type_SMF,
                     sess, AMF_UPDATE_SM_CONTEXT_N1_RELEASED, &param,
-                    amf_nsmf_pdu_session_build_update_sm_context);
+                    amf_nsmf_pdusession_build_update_sm_context);
 
             if (gsm_header->message_type ==
                     OGS_NAS_5GS_PDU_SESSION_RELEASE_COMPLETE) {
