@@ -26,6 +26,7 @@
 #include "nausf-handler.h"
 #include "nsmf-handler.h"
 #include "nudm-handler.h"
+#include "npcf-handler.h"
 #include "sbi-path.h"
 #include "amf-sm.h"
 
@@ -63,6 +64,8 @@ void gmm_state_de_registered(ogs_fsm_t *s, amf_event_t *e)
 
     switch (e->id) {
     case OGS_FSM_ENTRY_SIG:
+        AMF_UE_CLEAR_PAGING_INFO(amf_ue);
+        AMF_UE_CLEAR_N2_TRANSFER(amf_ue, pdu_session_resource_setup_request);
         CLEAR_AMF_UE_ALL_TIMERS(amf_ue);
         break;
     case OGS_FSM_EXIT_SIG:
@@ -125,6 +128,7 @@ static void common_register_state(ogs_fsm_t *s, amf_event_t *e)
 
         switch (nas_message->gmm.h.message_type) {
         case OGS_NAS_5GS_REGISTRATION_REQUEST:
+            ogs_info("Registration request");
             rv = gmm_handle_registration_request(
                     amf_ue, &nas_message->gmm.registration_request);
             if (rv != OGS_OK) {
@@ -169,6 +173,8 @@ static void common_register_state(ogs_fsm_t *s, amf_event_t *e)
             break;
 
         case OGS_NAS_5GS_SERVICE_REQUEST:
+            ogs_info("Service request");
+
             rv = gmm_handle_service_request(
                     amf_ue, &nas_message->gmm.service_request);
             if (rv != OGS_OK) {
@@ -178,7 +184,7 @@ static void common_register_state(ogs_fsm_t *s, amf_event_t *e)
             }
 
             if (!AMF_UE_HAVE_SUCI(amf_ue)) {
-                ogs_error("Service request : Unknown UE");
+                ogs_info("Service request : Unknown UE");
                 nas_5gs_send_service_reject(amf_ue,
                 OGS_5GMM_CAUSE_UE_IDENTITY_CANNOT_BE_DERIVED_BY_THE_NETWORK);
                 OGS_FSM_TRAN(s, gmm_state_exception);
@@ -206,6 +212,7 @@ static void common_register_state(ogs_fsm_t *s, amf_event_t *e)
         case OGS_NAS_5GS_IDENTITY_RESPONSE:
             CLEAR_AMF_UE_TIMER(amf_ue->t3570);
 
+            ogs_info("Identity response");
             rv = gmm_handle_identity_response(amf_ue,
                     &nas_message->gmm.identity_response);
             if (rv != OGS_OK) {
@@ -230,65 +237,6 @@ static void common_register_state(ogs_fsm_t *s, amf_event_t *e)
             OGS_FSM_TRAN(s, &gmm_state_authentication);
             break;
 
-#if 0
-        case OGS_NAS_5GS_TRACKING_AREA_UPDATE_REQUEST:
-            ogs_error("[%s] Tracking area update request",
-                    amf_ue->supi);
-            rv = gmm_handle_tau_request(
-                    amf_ue, &nas_message->gmm.tracking_area_update_request);
-            if (rv != OGS_OK) {
-                ogs_error("gmm_handle_tau_request() failed");
-                OGS_FSM_TRAN(s, gmm_state_exception);
-                break;
-            }
-
-            if (!AMF_UE_HAVE_SUCI(amf_ue)) {
-                ogs_warn("TAU request : Unknown UE");
-                nas_5gs_send_tau_reject(amf_ue,
-                    GMM_CAUSE_UE_IDENTITY_CANNOT_BE_DERIVED_BY_THE_NETWORK);
-                OGS_FSM_TRAN(s, &gmm_state_exception);
-                break;
-            }
-            break;
-            case AMF_EPS_TYPE_TAU_REQUEST:
-                procedureCode = e->ngap_code;
-
-                if (!SESSION_CONTEXT_IS_AVAILABLE(amf_ue)) {
-                    ogs_warn("No PDN Connection : UE[%s]", amf_ue->imsi_bcd);
-                    nas_5gs_send_tau_reject(amf_ue,
-                        GMM_CAUSE_UE_IDENTITY_CANNOT_BE_DERIVED_BY_THE_NETWORK);
-                    OGS_FSM_TRAN(s, gmm_state_exception);
-                    break;
-                }
-
-                if (!SECURITY_CONTEXT_IS_VALID(amf_ue)) {
-                    amf_s6a_send_air(amf_ue, NULL);
-                    OGS_FSM_TRAN(&amf_ue->sm, &gmm_state_authentication);
-                    break;
-                }
-
-                if (procedureCode == NGAP_ProcedureCode_id_initialUEMessage) {
-                    ogs_debug("    Iniital UE Message");
-                    if (amf_ue->nas_5gs.update.active_flag) {
-                        nas_5gs_send_tau_accept(amf_ue,
-                                NGAP_ProcedureCode_id_InitialContextSetup);
-                    } else {
-                        nas_5gs_send_tau_accept(amf_ue,
-                                NGAP_ProcedureCode_id_downlinkNASTransport);
-                        amf_send_release_access_bearer_or_ue_context_release(ran_ue);
-                    }
-                } else if (procedureCode == NGAP_ProcedureCode_id_uplinkNASTransport) {
-                    ogs_debug("    Uplink NAS Transport");
-                    nas_5gs_send_tau_accept(amf_ue,
-                            NGAP_ProcedureCode_id_downlinkNASTransport);
-                } else {
-                    ogs_fatal("Invalid Procedure Code[%d]", (int)procedureCode);
-                }
-                break;
-        case OGS_NAS_5GS_TRACKING_AREA_UPDATE_COMPLETE:
-            ogs_error("[%s] Tracking area update complete", amf_ue->supi);
-            break;
-#endif
         case OGS_NAS_5GS_5GMM_STATUS:
             ogs_warn("[%s] 5GMM STATUS : Cause[%d]", amf_ue->suci,
                     nas_message->gmm.gmm_status.gmm_cause);
@@ -296,6 +244,8 @@ static void common_register_state(ogs_fsm_t *s, amf_event_t *e)
             break;
 
         case OGS_NAS_5GS_DEREGISTRATION_REQUEST:
+            ogs_info("[%s] Deregistration request", amf_ue->supi);
+
             gmm_handle_deregistration_request(
                     amf_ue, &nas_message->gmm.deregistration_request_from_ue);
             OGS_FSM_TRAN(s, &gmm_state_de_registered);
@@ -318,6 +268,11 @@ static void common_register_state(ogs_fsm_t *s, amf_event_t *e)
                     amf_ue, &nas_message->gmm.ul_nas_transport);
             break;
 
+        case OGS_NAS_5GS_REGISTRATION_COMPLETE:
+            ogs_error("[%s] Registration complete in INVALID-STATE",
+                        amf_ue->supi);
+            break;
+
         default:
             ogs_error("Unknown message [%d]", nas_message->gmm.h.message_type);
         }
@@ -328,20 +283,32 @@ static void common_register_state(ogs_fsm_t *s, amf_event_t *e)
         case AMF_TIMER_T3513:
             if (amf_ue->t3513.retry_count >=
                     amf_timer_cfg(AMF_TIMER_T3513)->max_count) {
+                amf_sess_t *sess = NULL;
+
                 /* Paging failed */
                 ogs_warn("[%s] Paging failed. Stop", amf_ue->supi);
+
+                ogs_list_for_each(&amf_ue->sess_list, sess) {
+                    if (sess->paging.ongoing == true) {
+                        amf_sbi_send_n1_n2_failure_notify(
+                            sess, OpenAPI_n1_n2_message_transfer_cause_UE_NOT_RESPONDING);
+                    }
+                }
+
+                /* Clear Paging Info */
+                AMF_UE_CLEAR_PAGING_INFO(amf_ue);
+
+                /* Clear N2 Transfer */
+                AMF_UE_CLEAR_N2_TRANSFER(
+                        amf_ue, pdu_session_resource_setup_request);
+
+                /* Clear t3513 Timers */
                 CLEAR_AMF_UE_TIMER(amf_ue->t3513);
 
             } else {
                 amf_ue->t3513.retry_count++;
-                /*
-                 * If t3513 is timeout, the saved pkbuf is used.
-                 * We don't have to set CNDomain.
-                 * So, we just set CNDomain to 0
-                 */
-#if 0
-                ngap_send_paging(amf_ue, 0);
-#endif
+                /* If t3513 is timeout, the saved pkbuf is used.  */
+                ngap_send_paging(amf_ue);
             }
             break;
 
@@ -509,6 +476,8 @@ void gmm_state_authentication(ogs_fsm_t *s, amf_event_t *e)
             break;
 
         case OGS_NAS_5GS_DEREGISTRATION_REQUEST:
+            ogs_warn("[%s] Deregistration request", amf_ue->supi);
+
             gmm_handle_deregistration_request(
                     amf_ue, &nas_message->gmm.deregistration_request_from_ue);
             OGS_FSM_TRAN(s, &gmm_state_de_registered);
@@ -683,6 +652,9 @@ void gmm_state_security_mode(ogs_fsm_t *s, amf_event_t *e)
             ogs_kdf_nh_gnb(amf_ue->kamf, amf_ue->kgnb, amf_ue->nh);
             amf_ue->nhcc = 1;
 
+            /* Create New GUTI */
+            amf_ue_new_guti(amf_ue);
+
             amf_ue_sbi_discover_and_send(OpenAPI_nf_type_UDM, amf_ue, NULL,
                     amf_nudm_uecm_build_registration);
 
@@ -691,15 +663,9 @@ void gmm_state_security_mode(ogs_fsm_t *s, amf_event_t *e)
             } else if (amf_ue->nas.message_type ==
                         OGS_NAS_5GS_SERVICE_REQUEST) {
                 OGS_FSM_TRAN(s, &gmm_state_registered);
-#if 0
-            } else if (amf_ue->nas.message_type ==
-                        OGS_NAS_5GS_SERVICE_REQUEST ||
-                            amf_ue->nas.message_type ==
-                                AMF_EPS_TYPE_TAU_REQUEST) {
-                OGS_FSM_TRAN(s, &gmm_state_registered);
-#endif
             } else {
                 ogs_fatal("Invalid OGS_NAS_5GS[%d]", amf_ue->nas.message_type);
+                ogs_assert_if_reached();
             }
             break;
         case OGS_NAS_5GS_SECURITY_MODE_REJECT:
@@ -724,21 +690,13 @@ void gmm_state_security_mode(ogs_fsm_t *s, amf_event_t *e)
 
             OGS_FSM_TRAN(s, &gmm_state_authentication);
             break;
-        case OGS_NAS_5GS_SERVICE_REQUEST:
-            ogs_debug("[%s] Service request", amf_ue->supi);
-            nas_5gs_send_service_reject(amf_ue,
-                    OGS_5GMM_CAUSE_SECURITY_MODE_REJECTED_UNSPECIFIED);
-            OGS_FSM_TRAN(s, &gmm_state_exception);
-            break;
 
-#if 0
-        case OGS_NAS_5GS_TRACKING_AREA_UPDATE_REQUEST:
-            ogs_debug("Tracking area update request");
-            nas_5gs_send_tau_reject(amf_ue,
-                GMM_CAUSE_SECURITY_MODE_REJECTED_UNSPECIFIED);
+        case OGS_NAS_5GS_SERVICE_REQUEST:
+            ogs_info("[%s] Service request", amf_ue->supi);
+            nas_5gs_send_service_reject(amf_ue,
+                OGS_5GMM_CAUSE_UE_IDENTITY_CANNOT_BE_DERIVED_BY_THE_NETWORK);
             OGS_FSM_TRAN(s, &gmm_state_exception);
             break;
-#endif
 
         case OGS_NAS_5GS_5GMM_STATUS:
             ogs_warn("[%s] 5GMM STATUS : Cause[%d]",
@@ -747,6 +705,8 @@ void gmm_state_security_mode(ogs_fsm_t *s, amf_event_t *e)
             break;
 
         case OGS_NAS_5GS_DEREGISTRATION_REQUEST:
+            ogs_warn("[%s] Deregistration request", amf_ue->supi);
+
             gmm_handle_deregistration_request(
                     amf_ue, &nas_message->gmm.deregistration_request_from_ue);
             OGS_FSM_TRAN(s, &gmm_state_de_registered);
@@ -862,6 +822,7 @@ void gmm_state_initial_context_setup(ogs_fsm_t *s, amf_event_t *e)
             SWITCH(sbi_message->h.resource.component[1])
             CASE(OGS_SBI_RESOURCE_NAME_AM_DATA)
             CASE(OGS_SBI_RESOURCE_NAME_SMF_SELECT_DATA)
+            CASE(OGS_SBI_RESOURCE_NAME_UE_CONTEXT_IN_SMF_DATA)
                 if (sbi_message->res_status != OGS_SBI_HTTP_STATUS_OK) {
                     ogs_error("[%s] HTTP response error [%d]",
                             amf_ue->supi, sbi_message->res_status);
@@ -871,9 +832,41 @@ void gmm_state_initial_context_setup(ogs_fsm_t *s, amf_event_t *e)
                     break;
                 }
 
-                amf_nudm_sdm_handle_provisioned(amf_ue, sbi_message);
+                rv = amf_nudm_sdm_handle_provisioned(amf_ue, sbi_message);
+                if (rv != OGS_OK) {
+                    ogs_error("[%s] amf_nudm_sdm_handle_provisioned(%s) failed",
+                            amf_ue->supi, sbi_message->h.resource.component[1]);
+                    nas_5gs_send_gmm_reject(
+                            amf_ue, OGS_5GMM_CAUSE_5GS_SERVICES_NOT_ALLOWED);
+                    OGS_FSM_TRAN(&amf_ue->sm, &gmm_state_exception);
+                    break;
+                }
                 break;
-            CASE(OGS_SBI_RESOURCE_NAME_UE_CONTEXT_IN_SMF_DATA)
+
+            DEFAULT
+                ogs_error("Invalid resource name [%s]",
+                        sbi_message->h.resource.component[1]);
+                ogs_assert_if_reached();
+            END
+            break;
+
+        CASE(OGS_SBI_SERVICE_NAME_NPCF_AM_POLICY_CONTROL)
+            SWITCH(sbi_message->h.resource.component[0])
+            CASE(OGS_SBI_RESOURCE_NAME_POLICIES)
+                rv = amf_npcf_am_policy_control_handle_create(
+                        amf_ue, sbi_message);
+                if (rv != OGS_OK) {
+                    ogs_error("[%s] amf_npcf_am_policy_control_handle_create() "
+                            "failed", amf_ue->supi);
+                    OGS_FSM_TRAN(&amf_ue->sm, &gmm_state_exception);
+                    break;
+                }
+
+/*
+ * The usage of guti_present is changed
+ * This following code should be removed.
+ */
+#if 0
                 /*
                  * Issues #553
                  *
@@ -893,22 +886,17 @@ void gmm_state_initial_context_setup(ogs_fsm_t *s, amf_event_t *e)
                  */
                 if (amf_ue->guti_present == 0)
                     OGS_FSM_TRAN(&amf_ue->sm, &gmm_state_registered);
+#endif
 
-                /*
-                 * Do not use nas_5gs_send_registration_accept()
-                 * instead of nas_5gs_send_accept() here.
-                 *
-                 * nas_5gs_send_service_accept() could be used later.
-                 * The reason is why the design could be changed to handle this.
-                 *
-                 * So we'll use nas_5gs_send_accept() at this point.
-                 */
-                nas_5gs_send_accept(amf_ue);
+                /* If nas_5gs_send_service_accept() used, we need change it. */
+                ogs_assert(amf_ue->nas.message_type ==
+                        OGS_NAS_5GS_REGISTRATION_REQUEST);
+                nas_5gs_send_registration_accept(amf_ue);
                 break;
 
             DEFAULT
                 ogs_error("Invalid resource name [%s]",
-                        sbi_message->h.resource.component[1]);
+                        sbi_message->h.resource.component[0]);
                 ogs_assert_if_reached();
             END
             break;
@@ -916,7 +904,6 @@ void gmm_state_initial_context_setup(ogs_fsm_t *s, amf_event_t *e)
         DEFAULT
             ogs_error("Invalid service name [%s]", sbi_message->h.service.name);
             ogs_assert_if_reached();
-
         END
         break;
 
@@ -926,7 +913,10 @@ void gmm_state_initial_context_setup(ogs_fsm_t *s, amf_event_t *e)
 
         switch (nas_message->gmm.h.message_type) {
         case OGS_NAS_5GS_REGISTRATION_COMPLETE:
-            ogs_debug("[%s] Registration complete", amf_ue->supi);
+            ogs_info("[%s] Registration complete", amf_ue->supi);
+
+            /* Clear GUTI present */
+            amf_ue->guti_present = false;
 
             /*
              * TS24.501
@@ -978,6 +968,13 @@ void gmm_state_initial_context_setup(ogs_fsm_t *s, amf_event_t *e)
             OGS_FSM_TRAN(s, &gmm_state_authentication);
             break;
 
+        case OGS_NAS_5GS_SERVICE_REQUEST:
+            ogs_info("[%s] Service request", amf_ue->supi);
+            nas_5gs_send_service_reject(amf_ue,
+                OGS_5GMM_CAUSE_UE_IDENTITY_CANNOT_BE_DERIVED_BY_THE_NETWORK);
+            OGS_FSM_TRAN(s, &gmm_state_exception);
+            break;
+
         case OGS_NAS_5GS_5GMM_STATUS:
             ogs_warn("[%s] 5GMM STATUS : Cause[%d]",
                     amf_ue->supi, nas_message->gmm.gmm_status.gmm_cause);
@@ -985,6 +982,8 @@ void gmm_state_initial_context_setup(ogs_fsm_t *s, amf_event_t *e)
             break;
 
         case OGS_NAS_5GS_DEREGISTRATION_REQUEST:
+            ogs_warn("[%s] Deregistration request", amf_ue->supi);
+
             gmm_handle_deregistration_request(
                     amf_ue, &nas_message->gmm.deregistration_request_from_ue);
             OGS_FSM_TRAN(s, &gmm_state_de_registered);
@@ -1033,6 +1032,8 @@ void gmm_state_exception(ogs_fsm_t *s, amf_event_t *e)
 
     switch (e->id) {
     case OGS_FSM_ENTRY_SIG:
+        AMF_UE_CLEAR_PAGING_INFO(amf_ue);
+        AMF_UE_CLEAR_N2_TRANSFER(amf_ue, pdu_session_resource_setup_request);
         CLEAR_AMF_UE_ALL_TIMERS(amf_ue);
 
         amf_sbi_send_release_all_sessions(
@@ -1060,6 +1061,7 @@ void gmm_state_exception(ogs_fsm_t *s, amf_event_t *e)
 
         switch (nas_message->gmm.h.message_type) {
         case OGS_NAS_5GS_REGISTRATION_REQUEST:
+            ogs_info("Registration request");
             rv = gmm_handle_registration_request(
                     amf_ue, &nas_message->gmm.registration_request);
             if (rv != OGS_OK) {
