@@ -187,6 +187,9 @@ static void test1_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -614,6 +617,9 @@ static void test3_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -665,6 +671,9 @@ static void test3_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 0;
     sess->ul_nas_transport_param.s_nssai = 0;
 
+    sess->pdu_session_establishment_param.ssc_mode = 0;
+    sess->pdu_session_establishment_param.epco = 0;
+
     gsmbuf = testgsm_build_pdu_session_release_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -695,6 +704,9 @@ static void test3_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 0;
     sess->ul_nas_transport_param.s_nssai = 0;
 
+    sess->pdu_session_establishment_param.ssc_mode = 0;
+    sess->pdu_session_establishment_param.epco = 0;
+
     gsmbuf = testgsm_build_pdu_session_release_complete(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -775,6 +787,9 @@ static void test3_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -1019,6 +1034,9 @@ static void test4_func(abts_case *tc, void *data)
         sess->ul_nas_transport_param.dnn = 1;
         sess->ul_nas_transport_param.s_nssai = 1;
 
+        sess->pdu_session_establishment_param.ssc_mode = 1;
+        sess->pdu_session_establishment_param.epco = 1;
+
         gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
         ABTS_PTR_NOTNULL(tc, gsmbuf);
         gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -1061,6 +1079,9 @@ static void test4_func(abts_case *tc, void *data)
         sess->ul_nas_transport_param.dnn = 0;
         sess->ul_nas_transport_param.s_nssai = 0;
 
+        sess->pdu_session_establishment_param.ssc_mode = 0;
+        sess->pdu_session_establishment_param.epco = 0;
+
         gsmbuf = testgsm_build_pdu_session_release_request(sess);
         ABTS_PTR_NOTNULL(tc, gsmbuf);
         gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -1091,6 +1112,9 @@ static void test4_func(abts_case *tc, void *data)
         sess->ul_nas_transport_param.dnn = 0;
         sess->ul_nas_transport_param.s_nssai = 0;
 
+        sess->pdu_session_establishment_param.ssc_mode = 0;
+        sess->pdu_session_establishment_param.epco = 0;
+
         gsmbuf = testgsm_build_pdu_session_release_complete(sess);
         ABTS_PTR_NOTNULL(tc, gsmbuf);
         gmmbuf = testgmm_build_ul_nas_transport(sess,
