@@ -177,6 +177,9 @@ static void test1_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -815,6 +818,9 @@ static void test3_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -1238,6 +1244,9 @@ static void test4_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -1618,6 +1627,9 @@ static void test5_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -1730,6 +1742,9 @@ static void test5_func(abts_case *tc, void *data)
         OGS_NAS_5GS_REQUEST_TYPE_INITIAL;
     sess->ul_nas_transport_param.dnn = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -2001,6 +2016,9 @@ static void test6_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
