@@ -196,6 +196,9 @@ static void test1_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -251,6 +254,9 @@ static void test1_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -292,6 +298,9 @@ static void test1_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 0;
     sess->ul_nas_transport_param.s_nssai = 0;
 
+    sess->pdu_session_establishment_param.ssc_mode = 0;
+    sess->pdu_session_establishment_param.epco = 0;
+
     gsmbuf = testgsm_build_pdu_session_modification_complete(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -396,6 +405,9 @@ static void test1_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 0;
     sess->ul_nas_transport_param.s_nssai = 0;
 
+    sess->pdu_session_establishment_param.ssc_mode = 0;
+    sess->pdu_session_establishment_param.epco = 0;
+
     gsmbuf = testgsm_build_pdu_session_release_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -426,6 +438,9 @@ static void test1_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 0;
     sess->ul_nas_transport_param.s_nssai = 0;
 
+    sess->pdu_session_establishment_param.ssc_mode = 0;
+    sess->pdu_session_establishment_param.epco = 0;
+
     gsmbuf = testgsm_build_pdu_session_release_complete(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -653,6 +668,9 @@ static void test2_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -708,6 +726,9 @@ static void test2_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -749,6 +770,9 @@ static void test2_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 0;
     sess->ul_nas_transport_param.s_nssai = 0;
 
+    sess->pdu_session_establishment_param.ssc_mode = 0;
+    sess->pdu_session_establishment_param.epco = 0;
+
     gsmbuf = testgsm_build_pdu_session_modification_complete(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -1072,6 +1096,9 @@ static void test3_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -1127,6 +1154,9 @@ static void test3_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 1;
     sess->ul_nas_transport_param.s_nssai = 1;
 
+    sess->pdu_session_establishment_param.ssc_mode = 1;
+    sess->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -1168,6 +1198,9 @@ static void test3_func(abts_case *tc, void *data)
     sess->ul_nas_transport_param.dnn = 0;
     sess->ul_nas_transport_param.s_nssai = 0;
 
+    sess->pdu_session_establishment_param.ssc_mode = 0;
+    sess->pdu_session_establishment_param.epco = 0;
+
     gsmbuf = testgsm_build_pdu_session_modification_complete(sess);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess,
@@ -1477,6 +1510,9 @@ static void test4_func(abts_case *tc, void *data)
     sess5->ul_nas_transport_param.dnn = 1;
     sess5->ul_nas_transport_param.s_nssai = 1;
 
+    sess5->pdu_session_establishment_param.ssc_mode = 1;
+    sess5->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess5);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess5,
@@ -1496,6 +1532,9 @@ static void test4_func(abts_case *tc, void *data)
     sess6->ul_nas_transport_param.dnn = 1;
     sess6->ul_nas_transport_param.s_nssai = 1;
 
+    sess6->pdu_session_establishment_param.ssc_mode = 1;
+    sess6->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess6);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess6,
@@ -1576,6 +1615,9 @@ static void test4_func(abts_case *tc, void *data)
     sess6->ul_nas_transport_param.dnn = 0;
     sess6->ul_nas_transport_param.s_nssai = 0;
 
+    sess6->pdu_session_establishment_param.ssc_mode = 0;
+    sess6->pdu_session_establishment_param.epco = 0;
+
     gsmbuf = testgsm_build_pdu_session_modification_complete(sess6);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess6,
@@ -1809,6 +1851,9 @@ static void test5_func(abts_case *tc, void *data)
     sess5->ul_nas_transport_param.dnn = 1;
     sess5->ul_nas_transport_param.s_nssai = 1;
 
+    sess5->pdu_session_establishment_param.ssc_mode = 1;
+    sess5->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess5);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess5,
@@ -1844,6 +1889,9 @@ static void test5_func(abts_case *tc, void *data)
     sess6->ul_nas_transport_param.dnn = 1;
     sess6->ul_nas_transport_param.s_nssai = 1;
 
+    sess6->pdu_session_establishment_param.ssc_mode = 1;
+    sess6->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess6);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess6,
@@ -2073,6 +2121,9 @@ static void test6_func(abts_case *tc, void *data)
     sess5->ul_nas_transport_param.dnn = 1;
     sess5->ul_nas_transport_param.s_nssai = 1;
 
+    sess5->pdu_session_establishment_param.ssc_mode = 1;
+    sess5->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess5);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess5,
@@ -2108,6 +2159,9 @@ static void test6_func(abts_case *tc, void *data)
     sess6->ul_nas_transport_param.dnn = 1;
     sess6->ul_nas_transport_param.s_nssai = 1;
 
+    sess6->pdu_session_establishment_param.ssc_mode = 1;
+    sess6->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess6);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess6,
@@ -2135,6 +2189,9 @@ static void test6_func(abts_case *tc, void *data)
     sess5->ul_nas_transport_param.dnn = 1;
     sess5->ul_nas_transport_param.s_nssai = 1;
 
+    sess5->pdu_session_establishment_param.ssc_mode = 1;
+    sess5->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess5);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess5,
@@ -2162,6 +2219,9 @@ static void test6_func(abts_case *tc, void *data)
     sess6->ul_nas_transport_param.dnn = 1;
     sess6->ul_nas_transport_param.s_nssai = 1;
 
+    sess6->pdu_session_establishment_param.ssc_mode = 1;
+    sess6->pdu_session_establishment_param.epco = 1;
+
     gsmbuf = testgsm_build_pdu_session_establishment_request(sess6);
     ABTS_PTR_NOTNULL(tc, gsmbuf);
     gmmbuf = testgmm_build_ul_nas_transport(sess6,
